# -*- coding: utf-8 -*-

import os

from setuptools import setup
from setuptools import find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup(
    name='castle.cms',
    description='CastleCMS Plone distribution main package',
    long_description_content_type='text/x-rst',
    version='2.6.8',
    long_description='%s\n%s' % (
        read('README.rst'),
        read('HISTORY.rst')
    ),
    keywords="plone cms castle",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone",
        "Framework :: Plone :: 5.1",
        "Framework :: CastleCMS",
        "Framework :: CastleCMS :: Theme"
    ],
    author='Wildcard Corp',
    author_email='info@wildcardcorp.com',
    url='https://github.com/castlecms/castle.cms',
    license='GPL2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['castle'],
    install_requires=[
        # plone packages
        'Products.CMFPlone>=5.0.10',
        'mockup>=2.4.2.dev1',
        'plone.app.upgrade>=2.0.17',
        'plone.app.mosaic>=2.0.0.dev24',
        'plone.app.blocks>=10.0.0',
        'Products.PloneKeywordManager>=2.2.1',
        'plone.schemaeditor>=2.0.18',
        'plone.namedfile>=3.0.10',
        'plone.folder>=1.0.12',
        'AccessControl>=3.0.14',
        'z3c.relationfield>=0.7.1.dev0',
        'Products.ZCatalog>=3.0.3dev2',
        'plone.app.standardtiles>=2.0.0.dev0',
        'plone.stringinterp>=1.3.1',

        'Products.CMFPlacefulWorkflow',
        'plone.app.caching',
        'plone.app.dexterity',
        'plone.app.iterate',
        'plone.api',

        # castle
        'castle.theme>=1.0.6',

        # add-ons
        'collective.documentviewer>=5.0.4',
        'collective.elasticsearch>=2.0.5<3',
        'collective.celery>=1.1.4',

        # python
        'boto3>=1.9.222',
        'google-api-python-client>=1.4.2<2',
        'redis>=2.10.3<3',
        'requests>=2.7.0<3',
        'requests_oauthlib>=0.5.0<1',
        'oauth2client>=1.5.1<2',
        'redis>=2.10.5<3',
        'setuptools',
        'python-dateutil',
        'pyopenssl',
        'phonenumbers',
        'html2text',
        'pycountry',
        'tendo',
        'pylru',
        'sqlalchemy',

        # misc
        'z3c.unconfigure',
        'z3c.jbot',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'responses',
            'mock',
            'argon2_cffi',
            'plone.app.robotframework',
            'moto',
        ],
        'development': [
            'zest.releaser',
            'check-manifest',
        ],
        'profile': [
            'collective.profiler'
        ],
        'forms': [
            'collective.easyform [recaptcha]'
        ]
    },
    entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone

      [celery_tasks]
      castle = castle.cms.tasks

      [console_scripts]
      clean-plone-users = castle.cms.cron:clean_users
      social-counts = castle.cms.cron:social_counts
      archive-content = castle.cms.cron:archive_content
      content-popularity = castle.cms.cron:ga_popularity
      empty-trash = castle.cms.cron:empty_trash
      twitter-monitor = castle.cms.cron:twitter_monitor
      reindex-elasticsearch = castle.cms.cron:reindex_es
      send-forced-publish-alert = castle.cms.cron:forced_publish_alert
      castle-crawler = castle.cms.cron:crawler
      clean-drafts = castle.cms.cron:clean_drafts
      upgrade-sites = castle.cms.cron:upgrade_sites
      link-report = castle.cms.cron:link_report
      """,
    include_package_data=True,
    zip_safe=False,
)



from castle.cms.interfaces import ICastleLayer
from z3c.form.interfaces import ITerms
from z3c.form.interfaces import IWidget
from z3c.form.term import ChoiceTerms
from zope.component import adapter
from zope.interface import Interface
from zope.interface import implementer

from .field import IQueryChoice


@implementer(ITerms)
@adapter(
    Interface,
    ICastleLayer,
    Interface,
    IQueryChoice,
    IWidget)
def QueryChoiceTerms(context, request, form, field, widget):
    return ChoiceTerms(context, request, form, field, widget)



#
# Script to test upgrading from one version of castle/plone to another.
#
# How this works:
#  - read instance script for list of default path modules
#  - download new versions
#  - generate new instance scripts with custom versions
#  - create sites based on different versions
#  - run `upgrade-sites` script
#
import json
import os
import shutil
import stat
import subprocess

import requests


dir_path = os.path.dirname(os.path.realpath(__file__))


DOWNLOAD_LOCATIONS = {
    'castle.cms': 'https://github.com/castlecms/castle.cms/archive/{version}.zip',
    'Products.CMFPlone': 'https://github.com/plone/Products.CMFPlone/archive/{version}.zip',
    'plone.app.upgrade': 'https://github.com/plone/plone.app.upgrade/archive/{version}.zip',
    'mockup': 'https://github.com/plone/mockup/archive/{version}.zip'
}

PACKAGE_DIRECTORY = os.path.join(dir_path, 'packages')
SCRIPTS_DIR = os.path.join(dir_path, 'castle', 'cms', '_scripts')
CREATE_SITE_SCRIPT = os.path.join(SCRIPTS_DIR, 'create-site.py')

TEST_VERSIONS = {
    '2.0.45': {
        'registry': {
            'plone.backend_url': 'foobar.com'
        },
        'packages': {
            'Products.CMFPlone': '5.0.8',
            'mockup': '2.4.1',
            'plone.app.upgrade': '2.0.4'
        }
    },
    '2.1.0': {
        'packages': {
            'Products.CMFPlone': '5.0.8',
            'plone.app.upgrade': '2.0.4',
            'mockup': '2.4.1',
        }
    },
    '2.2.0': {
        'packages': {
            'Products.CMFPlone': '5.0.8',
            'plone.app.upgrade': '2.0.4'
        }
    },
    '2.3.0': {
        'packages': {
            'Products.CMFPlone': '5.0.8',
            'plone.app.upgrade': '2.0.4'
        }
    },
    '2.5.19': {
        'packages': {
            'Products.CMFPlone': '5.0.8',
            'plone.app.upgrade': '2.0.4'
        }
    }
}
PKG_EXTRA = '-py2.7.egg'


def output(txt):
    print('[\033[1;33mCastle Test\033[0m]: {}'.format(txt))


def download_package(name, version, env):
    if not os.path.exists(PACKAGE_DIRECTORY):
        os.mkdir(PACKAGE_DIRECTORY)

    result_filepath = os.path.join(
        PACKAGE_DIRECTORY, '{}-{}'.format(name, version))

    if os.path.exists(result_filepath + PKG_EXTRA):
        # already downloaded...
        return

    filepath = '{}.zip'.format(result_filepath)
    if not os.path.exists(filepath):
        url = DOWNLOAD_LOCATIONS[name].format(version=version)
        output('Downloading {}'.format(url))
        resp = requests.get(url)
        with open(filepath, 'wb') as fi:
            fi.write(resp.content)

    subprocess.check_output([
        'unzip',
        filepath,
        '-d',
        PACKAGE_DIRECTORY
    ])
    os.remove(filepath)

    # copy egg info over...
    original_egg_info = os.path.join(env['packages'][name]['path'], 'EGG-INFO')
    if os.path.exists(original_egg_info):
        new_egg_info = os.path.join(result_filepath, 'EGG-INFO')
        shutil.copytree(original_egg_info, new_egg_info)

    shutil.move(result_filepath, result_filepath + PKG_EXTRA)


def download_packages(env):
    for castle_version, data in TEST_VERSIONS.items():
        versions = data['packages']
        download_package('castle.cms', castle_version, env)
        for name, version in versions.items():
            download_package(name, version, env)


def get_package_environment():
    with open('bin/instance') as fi:
        fidata = fi.read()

    packages = {}
    for line in fidata.splitlines():
        if not line.startswith("  '"):
            continue
        path = line.strip().strip("',")
        path_name = path.split('/')[-1].rsplit('-', 1)[0]
        name, _, version = path_name.partition('-')
        packages[name] = {
            'version': version,
            'line': line,
            'path': path
        }
    return {
        'script': fidata,
        'packages': packages
    }


def get_bins(env):
    for castle_version in TEST_VERSIONS.keys():
        script = munge_py_env(env['script'], castle_version, env)
        yield castle_version, script


def munge_py_env(script, castle_version, env):
    packages = env['packages']
    script = script.replace(
        packages['castle.cms']['line'],
        "  '{}',".format(
            os.path.join(
                PACKAGE_DIRECTORY, 'castle.cms-{}{}'.format(castle_version, PKG_EXTRA))))
    for name, version in TEST_VERSIONS[castle_version]['packages'].items():
        script = script.replace(
            packages[name]['line'],
            "  '{}',".format(
                os.path.join(
                    PACKAGE_DIRECTORY, '{}-{}{}'.format(name, version, PKG_EXTRA))))
    script = script.replace(
        'parts/instance/etc/zope.conf',
        'parts/instance_{}/etc/zope.conf'.format(castle_version))
    return script


def build_instances(env):
    for version, instance_bin in get_bins(env):
        # build bin file
        filepath = 'bin/instance_{}'.format(version)
        with open(filepath, 'w') as fi:
            fi.write(instance_bin)
        st = os.stat(filepath)
        os.chmod(filepath, st.st_mode | stat.S_IEXEC)

        # copy new parts directory
        parts_path = os.path.join('parts', 'instance_{}'.format(version))
        if os.path.exists(parts_path):
            shutil.rmtree(parts_path)
        shutil.copytree('parts/instance', parts_path)

        zope_conf_filepath = os.path.join(parts_path, 'etc', 'zope.conf')
        with open(zope_conf_filepath) as fi:
            zope_conf = fi.read()
        zope_conf = zope_conf.replace('/parts/instance', '/parts/instance_{}'.format(version))
        with open(zope_conf_filepath, 'w') as fi:
            fi.write(zope_conf)

        interp_filepath = os.path.join(parts_path, 'bin', 'interpreter')
        with open(interp_filepath) as fi:
            interp = fi.read()
        interp = munge_py_env(interp, version, env)
        with open(interp_filepath, 'w') as fi:
            fi.write(interp)
        st = os.stat(interp_filepath)
        os.chmod(interp_filepath, st.st_mode | stat.S_IEXEC)


def build_sites(env):
    for version, instance_bin in get_bins(env):
        filepath = 'bin/instance_{}'.format(version)
        output('Creating site for {}'.format(version))
        env = os.environ.copy()
        env.update({
            'REGISTRY_DATA': json.dumps(
                TEST_VERSIONS[version].get('registry', {})
            )
        })
        process = subprocess.Popen([
            filepath,
            'run',
            CREATE_SITE_SCRIPT,
            '--site-id=Castle_{}'.format(version),
            '--delete=true'
        ], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
        out, err = process.communicate()
        if process.returncode != 0:
            # so we know why we errored out, return output here...
            print(out)
            print(err)
            raise Exception(
                'Invalid status code {}'.format(process.returncode))


def run():
    env = get_package_environment()
    download_packages(env)
    build_instances(env)
    build_sites(env)

    # test running upgrades now
    for version in TEST_VERSIONS.keys():
        output('Run upgrade for {}'.format(version))
        process = subprocess.Popen([
            'bin/upgrade-sites',
            '--site-id=Castle_{}'.format(version),
        ], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = process.communicate()
        if process.returncode != 0:
            print(out)
            print(err)
            raise Exception(
                'Invalid status code {}'.format(process.returncode))
        output('Upgrade successful for {}'.format(version))


if __name__ == '__main__':
    run()



from castle.cms.constants import CRAWLED_SITE_ES_DOC_TYPE
from castle.cms.utils import get_public_url
from collective.elasticsearch.es import ElasticSearchCatalog
from collective.elasticsearch.interfaces import IQueryAssembler
from DateTime import DateTime
from elasticsearch import TransportError
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import _getAuthenticatedUser
from Products.Five import BrowserView
from urlparse import urljoin
from urlparse import urlparse
from zope.component import getMultiAdapter
from zope.component import getUtility
import dateutil.parser

import datetime
import json
import Missing


def custom_json_handler(obj):
    if obj == Missing.Value:
        return None
    if type(obj) in (datetime.datetime, datetime.date):
        return obj.isoformat()
    if type(obj) == DateTime:
        return obj.ISO8601()
    return obj


def _one(val):
    if val and type(val) == list and len(val) == 1:
        val = val[0]
    return val


class Search(BrowserView):

    @property
    def options(self):
        search_types = [{
            'id': 'images',
            'label': 'Images',
            'query': {
                'portal_type': 'Image'
            }
        }, {
            'id': 'page',
            'label': 'Page',
            'query': {
                'portal_type': ['Document', 'Folder']
            }
        }]

        ptypes = api.portal.get_tool('portal_types')
        for type_id in ptypes.objectIds():
            if type_id in ('Link', 'Document', 'Folder'):
                continue
            _type = ptypes[type_id]
            if not _type.global_allow:
                continue
            search_types.append({
                'id': type_id.lower(),
                'label': _type.title,
                'query': {
                    'portal_type': type_id
                }
            })
        search_types.extend([{
            'id': 'video',
            'label': 'Video',
            'query': {
                'portal_type': 'Video'
            }
        }, {
            'id': 'audio',
            'label': 'Audio',
            'query': {
                'portal_type': 'Audio'
            }
        }])

        additional_sites = []
        es = ElasticSearchCatalog(api.portal.get_tool('portal_catalog'))
        if es.enabled:
            query = {
                "size": 0,
                "aggregations": {
                    "totals": {
                        "terms": {
                            "field": "domain"
                        }
                    }
                }
            }
            try:
                result = es.connection.search(
                    index=es.index_name,
                    doc_type=CRAWLED_SITE_ES_DOC_TYPE,
                    body=query)
                for res in result['aggregations']['totals']['buckets']:
                    site_name = res.get('key')
                    if '.' not in site_name or 'amazon' in site_name:
                        continue
                    additional_sites.append(site_name)
            except TransportError:
                pass

        parsed = urlparse(get_public_url())

        return json.dumps({
            'searchTypes': search_types,
            'additionalSites': [s for s in sorted(additional_sites)],
            'currentSiteLabel': parsed.netloc
        })

    @property
    def search_url(self):
        if api.user.is_anonymous():
            try:
                url = api.portal.get_registry_record('castle.searchurl')
                if url:
                    return url
            except Exception:
                pass
        return '%s/@@searchajax' % (
            self.context.absolute_url()
        )


_search_attributes = [
    'Title',
    'Description',
    'Subject',
    'contentType',
    'created',
    'modified',
    'effective',
    'hasImage',
    'is_folderish',
    'portal_type',
    'review_state',
    'url'
]

_valid_params = [
    'SearchableText',
    'portal_type',
    'Subject',
    'Subject:list',
    'after',
    'sort_on',
    'sort_order'
]


class SearchAjax(BrowserView):

    def __call__(self):
        self.catalog = api.portal.get_tool('portal_catalog')
        self.request.response.setHeader('Content-type', 'application/json')

        query = {}
        for name in _valid_params:
            real_name = name
            if real_name.endswith(':list'):
                real_name = real_name[:-len(':list')]
            if self.request.form.get(name):
                query[real_name] = self.request.form[name]
            elif self.request.form.get(name + '[]'):
                query[real_name] = self.request.form[name + '[]']

        if query.get('after'):
            if query.get('sort_on') not in ('effective', 'modified', 'created'):
                sort_on = query['sort_on'] = 'effective'
            else:
                sort_on = query['sort_on']
            try:
                date = dateutil.parser.parse(query.pop('after'))
                start = DateTime(date)
                query[sort_on] = {
                    'query': start,
                    'range': 'min'
                }
            except (KeyError, AttributeError, ValueError, TypeError):
                pass

        try:
            page_size = int(self.request.form.get('pageSize'))
        except Exception:
            page_size = 20
        page_size = min(page_size, 50)
        try:
            page = int(self.request.form.get('page'))
        except Exception:
            page = 1
        catalog = api.portal.get_tool('portal_catalog')
        es = ElasticSearchCatalog(catalog)
        if es.enabled:
            return self.get_es_results(page, page_size, query)
        else:
            return self.get_results(page, page_size, query)

    def get_results(self, page, page_size, query):
        # regular plone search
        site_path = '/'.join(self.context.getPhysicalPath())
        start = (page - 1) * page_size
        end = start + page_size
        catalog = api.portal.get_tool('portal_catalog')
        raw_results = catalog(**query)
        items = []

        registry = getUtility(IRegistry)
        view_types = registry.get('plone.types_use_view_action_in_listings', [])

        for brain in raw_results[start:end]:
            attrs = {}
            for key in _search_attributes:
                attrs[key] = getattr(brain, key, None)
            url = base_url = brain.getURL()
            if brain.portal_type in view_types:
                url += '/view'
            attrs.update({
                'path': brain.getPath()[len(site_path):],
                'base_url': base_url,
                'url': url
            })
            items.append(attrs)

        return json.dumps({
            'count': len(raw_results),
            'results': items,
            'page': page,
            'suggestions': []
        }, default=custom_json_handler)

    def get_es_results(self, page, page_size, query):
        start = (page - 1) * page_size
        site_path = '/'.join(self.context.getPhysicalPath())
        base_site_url = self.context.absolute_url()

        results = suggestions = []
        count = 0
        if len(query) > 0:
            results = self.search_es(query, start, page_size)
            count = results['hits']['total']
            try:
                suggestions = results['suggest']['SearchableText'][0]['options']
            except Exception:
                suggestions = []
            results = results['hits']['hits']

        registry = getUtility(IRegistry)
        view_types = registry.get('plone.types_use_view_action_in_listings', [])

        items = []
        for res in results:
            fields = res['fields']

            attrs = {}
            for key in _search_attributes:
                val = fields.get(key)
                attrs[key] = _one(val)

            if 'url' not in fields:
                path = fields.get('path.path', [''])
                path = path[0]
                path = path[len(site_path):]
                url = base_url = urljoin(base_site_url + '/', path.lstrip('/'))
                if attrs.get('portal_type') in view_types:
                    url += '/view'
            else:
                url = base_url = _one(fields['url'])
                parsed = urlparse(url)
                path = parsed.path

            attrs.update({
                'review_state': 'published',
                'score': res['_score'],
                'path': path,
                'base_url': base_url,
                'url': url
            })

            items.append(attrs)

        return json.dumps({
            'count': count,
            'results': items,
            'page': page,
            'suggestions': suggestions
        }, default=custom_json_handler)

    def search_es(self, query, start, size):
        user = _getAuthenticatedUser(self.catalog)
        query['allowedRolesAndUsers'] = self.catalog._listAllowedRolesAndUsers(user)

        es = ElasticSearchCatalog(self.catalog)
        qassembler = getMultiAdapter((self.request, es), IQueryAssembler)
        dquery, sort = qassembler.normalize(query)
        equery = qassembler(dquery)

        doc_type = es.doc_type
        if 'searchSite' in self.request.form:
            doc_type = CRAWLED_SITE_ES_DOC_TYPE
            equery = {
                'filtered': {
                    'filter': {
                        "term": {
                            "domain": self.request.form['searchSite']
                        }
                    },
                    'query': equery['function_score']['query']['filtered']['query']
                }
            }

        query = {
            'query': equery,
            "suggest": {
                "SearchableText": {
                    "text": query.get('SearchableText', ''),
                    "term": {
                        "field": "SearchableText"
                    }
                }
            },
            'sort': sort
        }

        query_params = {
            'from_': start,
            'size': size,
            'fields': ','.join(_search_attributes) + ',path.path'
        }

        return es.connection.search(index=es.index_name,
                                    doc_type=doc_type,
                                    body=query,
                                    **query_params)



from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.resources.browser.cook import cookWhenChangingSettings


PROFILE_ID = 'profile-castle.cms:2_1_0'


def upgrade(context, logger=None):
    # This value is being converted to a tuple, hold onto it or it will get
    # overwritten
    backend_url = api.portal.get_registry_record('plone.backend_url') or None

    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)
    cookWhenChangingSettings(api.portal.get())

    if isinstance(backend_url, tuple):
        pass
    elif isinstance(backend_url, unicode):
        api.portal.set_registry_record('plone.backend_url', (backend_url,))
    else:
        api.portal.set_registry_record('plone.backend_url', ())



# -*- coding: utf-8 -*-
from castle.cms.tests.utils import render_tile
import json
import unittest

from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms.tiles.dynamic import CACHE_KEY
from castle.cms.tiles.dynamic import FIELD_TYPE_MAPPING
from castle.cms.tiles.dynamic import get_tile_manager
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.theming.interfaces import IThemeSettings
from plone.app.theming.utils import applyTheme
from plone.app.theming.utils import createThemeFromTemplate
from plone.app.theming.utils import getTheme
from plone.registry.interfaces import IRegistry
from plone.resource.utils import queryResourceDirectory
from zope.component import getUtility


class TestDynamicTiles(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

        # part of setup is to always copy the theme to new one
        createThemeFromTemplate('foobar', '', 'castle.theme')
        themeData = getTheme('foobar')
        applyTheme(themeData)
        registry = getUtility(IRegistry)
        theme_settings = registry.forInterface(IThemeSettings, False)
        theme_settings.enabled = True

        directory = queryResourceDirectory('theme', 'foobar')
        directory.makeDirectory('tiles')
        self.tiles_directory = directory['tiles']

        # clear cache
        for key in list(self.request.environ.keys())[:]:
            if key.startswith(CACHE_KEY):
                del self.request.environ[key]

    def _make_tile(self, config, tile_id='foobartile', template='<div>Hello World!</div>'):
        self.tiles_directory.makeDirectory('foobartile')
        tile_dir = self.tiles_directory['foobartile']
        tile_dir.writeFile('config.json', json.dumps(config))
        tile_dir.writeFile('template.html', template)

    def test_provide_tile_in_theme(self):
        self._make_tile({
            "title": "Foobar",
            "category": "media",
            "weight": 200,
            "fields": [{
                "name": "foobar",
                "title": "Text",
                "required": False
            }]
        })

        mng = get_tile_manager(self.request)
        self.assertEqual(len(mng.get_tiles()), 1)

        tile = mng.get_tile('foobartile')
        self.assertIsNotNone(tile)

        self.assertEqual(len(mng.get_tile_fields('foobartile')), 1)

        iface = mng.get_schema('foobartile')
        self.assertEqual(iface.names(), ['foobar'])

    def test_all_fields(self):
        fields = []
        for idx, type_id in enumerate(FIELD_TYPE_MAPPING.keys()):
            fields.append({
                "name": "foobar{}".format(idx),
                "title": "Foobar {}".format(idx),
                "required": False,
                "type": type_id
            })

        self._make_tile({
            "title": "Foobar",
            "category": "media",
            "weight": 200,
            "fields": fields
        })
        mng = get_tile_manager(self.request)

        self.assertEqual(
            len(mng.get_tile_fields('foobartile')),
            len(FIELD_TYPE_MAPPING))

        iface = mng.get_schema('foobartile')
        self.assertEqual(len(iface.names()), len(FIELD_TYPE_MAPPING))

    def test_render_tile(self):
        self._make_tile({
            "title": "Foobar",
            "category": "media",
            "fields": [{
                "name": "foobar",
                "title": "Text",
                "required": False
            }]
        }, template='<div>Hello ${data/foobar}</div>')
        data = {
            'tile_id': 'foobartile',
            'foobar': 'Foobar!'
        }

        result = render_tile(self.request, self.portal,
                             'castle.cms.dynamic', data)
        self.assertIn('Hello Foobar!', result)



# -*- coding: utf-8 -*-
import boto3
import botocore
from castle.cms.browser import content
import castle.cms.files.aws as aws
from castle.cms.files.duplicates import DuplicateDetector
from castle.cms.testing import (
    CASTLE_PLONE_INTEGRATION_TESTING,
    CASTLE_PLONE_FUNCTIONAL_TESTING,
)
from persistent.mapping import PersistentMapping
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.uuid.interfaces import IUUID
from moto import mock_s3
from zope.annotation.interfaces import IAnnotations

from io import BytesIO
import json
from time import time
import unittest
from urllib.parse import quote_plus


class TestDuplicateDetector(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_register(self):
        dd = DuplicateDetector()
        hash_ = 'foobar'
        doc = api.content.create(
            type='Document',
            id='folder',
            container=self.portal)
        dd.register(doc, hash_)

        self.assertEqual(IUUID(dd.get_object(hash_)), IUUID(doc))


def upload_file_to_castle(self):
    self.request.form.update({
        'action': 'chunk-upload',
        'chunk': '1',
        'chunkSize': 1024,
        'totalSize': 1024 * 5,
        'file': BytesIO('X' * 1024),
        'name': 'foobar.bin'
    })
    cc = content.Creator(self.portal, self.request)
    data = json.loads(cc())
    self.request.form.update({
        'id': data['id']
    })
    for idx in range(4):
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': str(idx + 2),
            'file': BytesIO('X' * 1024)
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
    return api.content.get(path='/file-repository/foobar.bin')


class TestAWS(unittest.TestCase):
    layer = CASTLE_PLONE_FUNCTIONAL_TESTING

    def setUp(self):
        self.test_access_key = u'AKIAIOSFODNN7EXAMPLE'
        self.test_secret_key = u'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'
        self.test_bucket_name = u'castletest'
        # possibly useful to set under some condition...
        # self.test_bucket_endpoint = u'http://localhost:9000/'
        self.test_base_url = u'https://localhost.localdomain/'  # used for test swap
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))
        api.portal.set_registry_record('castle.aws_s3_key', self.test_access_key)
        api.portal.set_registry_record('castle.aws_s3_secret', self.test_secret_key)
        api.portal.set_registry_record('castle.aws_s3_bucket_name', self.test_bucket_name)
        # possibly useful to set under some condition...
        # api.portal.set_registry_record('castle.aws_s3_host_endpoint', self.test_bucket_endpoint)
        api.portal.set_registry_record('castle.aws_s3_base_url', self.test_base_url)

    def tearDown(self):
        pass

    @mock_s3
    def test_get_bucket(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket='castletest')

        s3, bucket = aws.get_bucket("castletest")
        self.assertIsNotNone(s3)
        self.assertIsNotNone(bucket)

    @mock_s3
    def test_move_file_private(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket='castletest')

        fileOb = upload_file_to_castle(self)

        s3, bucket = aws.get_bucket("castletest")
        uid = IUUID(fileOb)
        key = aws.KEY_PREFIX + uid

        # before move, there should be none of the meta information about the
        # moved version of the file
        self.assertFalse(hasattr(fileOb.file, 'original_filename'))
        self.assertFalse(hasattr(fileOb.file, 'original_size'))
        self.assertFalse(hasattr(fileOb.file, 'original_content_type'))

        # before move, there should be no annotations
        annotations = IAnnotations(fileOb)
        self.assertIsNone(annotations.get(aws.STORAGE_KEY, None))

        # before move, there should be no file in s3
        self.assertRaises(
            botocore.exceptions.ClientError,
            lambda: s3.meta.client.head_object(Bucket=bucket.name, Key=key))

        aws.move_file(fileOb)
        fileOb = api.content.get(path='/file-repository/foobar.bin')

        # after move, there should be additional meta information on the
        # file object
        self.assertTrue(hasattr(fileOb.file, 'original_filename'))
        self.assertTrue(hasattr(fileOb.file, 'original_size'))
        self.assertTrue(hasattr(fileOb.file, 'original_content_type'))

        # after move, there should be annotations on the object
        annotations = IAnnotations(fileOb)
        url = annotations[aws.STORAGE_KEY].get('url', None)
        expires_in = annotations[aws.STORAGE_KEY].get('expires_in', None)
        generated_on = annotations[aws.STORAGE_KEY].get('generated_on', None)
        self.assertIsNotNone(url)
        self.assertIsNotNone(expires_in)
        self.assertIsNotNone(generated_on)
        self.assertEqual(expires_in, aws.EXPIRES_IN)

        # after move, there should be no error from getting head information of
        # the uploaded object
        s3.meta.client.head_object(Bucket=bucket.name, Key=key)

    @mock_s3
    def test_move_file_public(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket='castletest')

        fileOb = upload_file_to_castle(self)

        api.content.transition(fileOb, 'publish')

        s3, bucket = aws.get_bucket("castletest")
        uid = IUUID(fileOb)
        key = aws.KEY_PREFIX + uid

        # before move, there should be none of the meta information about the
        # moved version of the file
        self.assertFalse(hasattr(fileOb.file, 'original_filename'))
        self.assertFalse(hasattr(fileOb.file, 'original_size'))
        self.assertFalse(hasattr(fileOb.file, 'original_content_type'))

        # before move, there should be no annotations
        annotations = IAnnotations(fileOb)
        self.assertIsNone(annotations.get(aws.STORAGE_KEY, None))

        # before move, there should be no file in s3
        self.assertRaises(
            botocore.exceptions.ClientError,
            lambda: s3.meta.client.head_object(Bucket=bucket.name, Key=key))

        aws.move_file(fileOb)
        fileOb = api.content.get(path='/file-repository/foobar.bin')

        # after move, there should be additional meta information on the
        # file object
        self.assertTrue(hasattr(fileOb.file, 'original_filename'))
        self.assertTrue(hasattr(fileOb.file, 'original_size'))
        self.assertTrue(hasattr(fileOb.file, 'original_content_type'))

        # after move, there should be annotations on the object
        annotations = IAnnotations(fileOb)
        url = annotations[aws.STORAGE_KEY].get('url', None)
        expires_in = annotations[aws.STORAGE_KEY].get('expires_in', None)
        generated_on = annotations[aws.STORAGE_KEY].get('generated_on', None)
        self.assertIsNotNone(url)
        self.assertIsNotNone(expires_in)
        self.assertIsNotNone(generated_on)
        self.assertEqual(expires_in, 0)

        # after move, there should be a file in s3, and checking for it should
        # produce no error
        s3.meta.client.head_object(Bucket=bucket.name, Key=key)

    @mock_s3
    def test_delete_file(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket='castletest')

        fileOb = upload_file_to_castle(self)

        s3, bucket = aws.get_bucket(s3_bucket='castletest')
        uid = IUUID(fileOb)
        key = aws.KEY_PREFIX + uid

        aws.move_file(fileOb)

        # before the delete operation, the file should exist
        # on s3, and this statement should not raise an exception
        s3.meta.client.head_object(Bucket=bucket.name, Key=key)

        aws.delete_file(uid)

        # after the delete operation, the file should not exist
        # on s3, but should still exist in plone (even if it
        # has no data...apparently...admittedly a place of possible
        # improvement?)
        self.assertRaises(
            botocore.exceptions.ClientError,
            lambda: s3.meta.client.head_object(Bucket=bucket.name, Key=key))

    @mock_s3
    def test_uploaded(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket='castletest')

        fileOb = upload_file_to_castle(self)

        self.assertFalse(aws.uploaded(fileOb))
        aws.move_file(fileOb)
        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertTrue(aws.uploaded(fileOb))

    @mock_s3
    def test_swap_url(self):
        awsurl = 'https://s3-us-gov-west-1.amazonaws.com/bucketname/archives/path/to/resource'
        baseurl = 'http://foo.com/'
        swappedurl = 'http://foo.com/archives/path/to/resource'

        resulturl = aws.swap_url(awsurl, base_url=baseurl)
        self.assertEqual(swappedurl, resulturl)

    @mock_s3
    def test_get_url_public_or_notexpired(self):
        # this is creating a bucket in the moto/mock s3 service
        bucket = 'castletest'
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket=bucket)

        fileOb = upload_file_to_castle(self)
        api.content.transition(fileOb, 'publish')
        aws.move_file(fileOb)
        fileOb = api.content.get(path='/file-repository/foobar.bin')

        resulturl = aws.get_url(fileOb)
        # the url should be the configured base url (from configuration registry)
        # with the key appended to it.
        # the bucketname should not be a part of the url. this should be stripped.
        # the key should also be sent through quote_plus, which basically makes it
        # a single path segment... this is okay with boto3 and the aws s3 api, and
        # possibly recommended to make sure your key's are passed to the api safely.
        key = quote_plus(aws.KEY_PREFIX + IUUID(fileOb))
        # wrap this in str before decode because of futures import
        expectedurl = str(self.test_base_url + key).decode('utf-8')
        self.assertEqual(resulturl, expectedurl)

    @mock_s3
    def test_get_url_private_expired(self):
        # this is creating a bucket in the moto/mock s3 service
        s3conn = boto3.resource('s3')
        s3conn.create_bucket(Bucket='castletest')

        fileOb = upload_file_to_castle(self)
        aws.move_file(fileOb)
        fileOb = api.content.get(path='/file-repository/foobar.bin')

        # move the generated further into the past
        annotations = IAnnotations(fileOb)
        info = annotations.get(aws.STORAGE_KEY, PersistentMapping())
        newgeneratedon = time() - aws.EXPIRES_IN - 1000
        info.update({
            'generated_on': newgeneratedon,
        })
        annotations[aws.STORAGE_KEY] = info

        resulturl = aws.get_url(fileOb)
        self.assertTrue(resulturl.startswith(self.test_base_url))

        fileOb = api.content.get(path='/file-repository/foobar.bin')
        annotations = IAnnotations(fileOb)
        info = annotations.get(aws.STORAGE_KEY, PersistentMapping())
        self.assertNotEqual(info["generated_on"], newgeneratedon)
        self.assertEqual(info["expires_in"], aws.EXPIRES_IN)



# -*- coding: utf-8 -*-
from Products.PloneKeywordManager.tests.test_controlpanel import ControlPanelTestCase
from Products.PloneKeywordManager.tests.test_dexterity import DexterityContentTestCase
from Products.PloneKeywordManager.tests.test_non_ascii_keywords import NonAsciiKeywordsTestCase
from Products.PloneKeywordManager.tests.test_setup import InstallTestCase
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING


class ControlPanel(ControlPanelTestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING


class DexterityContent(DexterityContentTestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING


class NonAsciiKeywords(NonAsciiKeywordsTestCase):
    # layer = CASTLE_PLONE_INTEGRATION_TESTING #fails for some reason?
    pass


class Install(InstallTestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING



import json
import time

from castle.cms import _, authentication, cache, texting
from castle.cms.interfaces import (IAuthenticator, ISecureLoginAllowedView,
                                   ISiteSchema)
from castle.cms.utils import (get_managers, send_email, strings_differ,
                              is_backend)
from plone import api
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from zope.component import getMultiAdapter, getUtility
from zope.component.interfaces import ComponentLookupError
from zope.i18n import translate
from zope.interface import implements


class SecureLoginView(BrowserView):
    implements(ISecureLoginAllowedView)

    def __init__(self, context, request):
        super(SecureLoginView, self).__init__(context, request)
        self.auth = self.authenticator = getMultiAdapter(
            (context, request), IAuthenticator)
        self.state_map = {
            self.auth.REQUESTING_AUTH_CODE: self.send_authorization,
            self.auth.CHECK_CREDENTIALS: self.login,
            self.auth.COUNTRY_BLOCKED: self.country_blocked
        }

    def __call__(self):
        state = self.auth.get_secure_flow_state()
        if self.request.REQUEST_METHOD == 'POST':
            self.request.response.setHeader('Content-type', 'application/json')
            if not self.auth.two_factor_enabled and state == self.auth.REQUESTING_AUTH_CODE:
                state = self.auth.CHECK_CREDENTIALS
                self.auth.set_secure_flow_state(state)
            if state in self.state_map:
                return self.state_map[state]()
            else:
                self.request.response.setStatus(403)
                return json.dumps({
                    'reason': 'Something went wrong.  Try again later.'
                })  # this shouldn't happen, state will expire.
        else:
            if state:
                self.auth.expire_secure_flow_state()
            if self.auth.two_factor_enabled:
                initial_state = self.auth.REQUESTING_AUTH_CODE
            else:
                initial_state = self.auth.CHECK_CREDENTIALS
            self.auth.set_secure_flow_state(initial_state)

            if self.request.cookies.get('castle_session_id', None) == 'test_session':
                return 'test-view'

            self.request.response.setHeader('X-Theme-Applied', True)
            return self.index()

    @property
    def username(self):
        return self.request.form.get('username', None)

    @property
    def from_backend_url(self):
        return is_backend(self.request)

    def scrub_backend(self):
        if not self.auth.is_zope_root and self.from_backend_url:
            return api.portal.get_registry_record('plone.scrub_title_logo_to_backend_login')
        return False

    def get_country_header(self):
        return (
            self.request.get_header('Cf-Ipcountry') or
            self.request.get_header('Country'))

    def country_blocked(self):
        if self.request.form.get('apiMethod', None) == 'request_country_exception':
            user = api.user.get(username=self.username)
            cache_key = self.auth.get_country_exception_cache_key(user.getId)
            try:
                request_data = cache.get(cache_key)
            except KeyError:
                request_data = None

            if not request_data:
                self.request_country_exception()
            else:
                if request_data.get('granted', False):
                    self.auth.set_secure_flow_state(self.auth.CHECK_CREDENTIALS)
                    return json.dumps({
                        'success': True,
                        'messsage': 'Your request has already been granted, try logging in again.'
                                    ' Requests expire 12 hours after issue.'
                    })
                else:
                    return json.dumps({
                        'success': True,
                        'message': 'Your request has been issued, but not yet granted.'
                                   ' Requests expire 12 hours after issue.'
                    })

    def request_country_exception(self):
        req_country = self.get_country_header()
        user = api.user.get(username=self.username)
        data = self.auth.issue_country_exception_request(user, req_country)

        email_subject = "Country block exception request(Site: %s)" % (
            api.portal.get_registry_record('plone.site_title'))

        for admin_user in get_managers():
            if admin_user.getId() == user.getId():
                # it could be an admin, do not allow him to authorize himself
                continue

            admin_email = admin_user.getProperty('email')

            email_data = data.copy()
            email_data.update({
                'admin_name': (admin_user.getProperty('fullname') or
                               admin_user.getUserName()),
                'name': (user.getProperty('fullname') or user.getUserName()),
                'auth_link': '{}/authorize-country-login?code={}&userid={}'.format(
                    self.context.absolute_url(),
                    data['code'],
                    user.getId()
                )
            })
            email_html = '''
<p>
Hello {admin_name},
</p>
<p>
There has been a request to suspend country login restrictions for this site.

<p>
The user requesting this access logged this information:
</p>
<ul>
<li><b>Username</b>: {username}</li>
<li><b>Name</b>: {name}</li>
<li><b>Country</b>: {country}</li>
<li><b>IP</b>: {ip}</li>
</ul>

<p>The user has 12 hours between after authorization has been giving to login
   with the temporary access</p>

<p>If you'd like to authorize this user, please click this link:
   <a href="{auth_link}">authorize {username}</a>
</p>'''.format(**email_data)
            send_email(admin_email, email_subject, html=email_html)

        return json.dumps({
            'success': True,
            'message': 'Successfully requested country exception.',
            'messageType': 'info'
        })

    def login(self):
        logged_in = False

        if not self.username:
            self.request.response.setHeader('X-Theme-Applied', True)
            self.request.response.setHeader('Content-type', 'text/html')
            return self.index()

        if self.auth.two_factor_enabled:
            if self.request.form.get('apiMethod', None) == 'resendAuth':
                self.auth.set_secure_flow_state(self.auth.REQUESTING_AUTH_CODE)
                return json.dumps({
                    'success': True,
                    'message': 'You may request another auth code.'
                })
            code = self.request.form.get('code')
            if not self.auth.authorize_2factor(self.username, code):
                return json.dumps({
                    'success': False,
                    'message': 'Two Factor is enabled, code not authorized.'
                })

        if hasattr(self.context, 'portal_registry'):
            backend_urls = self.context.portal_registry['plone.backend_url']
            only_allow_login_to_backend_urls = self.context.portal_registry['plone.only_allow_login_to_backend_urls']  # noqa
            portal_url = api.portal.get().absolute_url()
            bad_domain = only_allow_login_to_backend_urls and \
                         len(backend_urls) > 0 and \
                         portal_url.rstrip('/') not in backend_urls and \
                         portal_url.rstrip('/') + '/' not in backend_urls
            if bad_domain:
                return json.dumps({
                    'success': False,
                    'message': translate(_(
                        u'description_bad_login_domain',
                        default=u'You are attempting to log into this site from the wrong domain; contact your site administrator for assistance.'  # noqa
                    ))
                })

        try:
            logged_in, user = self.auth.authenticate(
                username=self.username,
                password=self.request.form.get('password'),
                country=self.get_country_header(),
                login=True)
        except authentication.AuthenticationMaxedLoginAttempts:
            return json.dumps({
                'success': False,
                'message': 'You have reached the max number of login attempts '
                           'for a period.'
            })
        except authentication.AuthenticationUserDisabled:
            return json.dumps({
                'success': False,
                'message': 'User is disabled.'
            })
        except authentication.AuthenticationCountryBlocked:
            self.auth.set_secure_flow_state(self.auth.COUNTRY_BLOCKED)
            return json.dumps({
                'success': False,
                'countryBlocked': True,
                'message': 'User login blocked. The country you are logging '
                           'in from is blocked.'
            })
        except authentication.AuthenticationPasswordResetWindowExpired:
            return json.dumps({
                'success': False,
                'message': 'User login disabled. Password reset request not '
                           'fullfilled in required time period.'
            })

        if not self.auth.is_zope_root and user:
            pw_expired = user.getProperty(
                'reset_password_required', False)
            if pw_expired:
                resp = {
                    'success': True,
                    'message': 'Your password has expired. Change it within 24 hours, or this account will be locked.',  # noqa
                    'changePasswordRequired': True,
                    'changePasswordUrl': api.portal.get().absolute_url() + '/@@change-password',
                }
                return json.dumps(resp)

        if logged_in:
            self.auth.expire_secure_flow_state()
            resp = {
                'success': True,
            }
            try:
                with api.env.adopt_user(user=user):
                    resp['authenticator'] = createToken()
                    return json.dumps(resp)
            except Exception:
                resp['authenticator'] = createToken()
                return json.dumps(resp)

        return json.dumps({
            'success': False,
            'message': 'Login failed.'
        })

    def authorize_code(self):
        code = self.request.form.get('code')
        if self.authenticator.authorize_2factor(self.username, code):
            new_state = self.auth.CHECK_CREDENTIALS
            self.auth.set_secure_flow_state(new_state)
            return json.dumps({
                'success': True,
                'message': 'Authorization code verified.',
                'state': new_state
            })
        else:
            return json.dumps({
                'success': False,
                'message': 'Failed to authorize code'
            })

    def send_authorization(self):
        auth_type = self.request.form.get('authType') or 'email'
        if auth_type == 'email':
            self.send_auth_email()
        elif auth_type == 'sms':
            self.send_auth_text()

        # send_* return false when a root acl_user tries to log in to a site
        # with 2factor enabled.  They need to log in at the root.
        # For now, responding the same regardless for security reasons.

        new_state = self.auth.CHECK_CREDENTIALS
        self.auth.set_secure_flow_state(new_state)
        return json.dumps({
            'success': True,
            'message': 'Authorization code sent to provided username.',
            'state': new_state
        })

    def send_auth_email(self):
        email = None
        user = api.user.get(username=self.username)
        if user:
            email = user.getProperty('email')
        if not email:
            return

        code = self.auth.issue_2factor_code(self.username)
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema,
                                              prefix="plone",
                                              check=False)
        html = """
<p>
    You have requested to authorize access to {title}.
</p>
<p>
    You authorization code is: {code}
</p>""".format(title=site_settings.site_title,
               code=code)

        send_email(
            [email], "Authorization code(%s)" % site_settings.site_title,
            html=html)

        return True

    def send_auth_text(self):
        phone = None
        user = api.user.get(username=self.username)
        if user:
            phone = user.getProperty('phone_number')

        if not phone:
            return

        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema,
                                              prefix="plone",
                                              check=False)

        code = self.auth.issue_2factor_code(self.username)

        text_message = '{}: phone verification code: {}'.format(
            site_settings.site_title, code)

        return texting.send(text_message, phone)

    def get_registry(self):
        try:
            return getUtility(IRegistry)
        except ComponentLookupError:
            pass

    def get_tool(self, name):
        if self.auth.is_zope_root:
            return getattr(self.context, name, None)
        else:
            return api.portal.get_tool(name)

    def options(self):
        return json.dumps(self.auth.get_options())


class LoginExceptionApprovalView(BrowserView):
    implements(ISecureLoginAllowedView)

    message = 'Incorrect code for country exception.'
    success = False

    def __call__(self):
        auth = self.authenticator = getMultiAdapter(
            (self.context, self.request), IAuthenticator)

        userid = self.request.form.get('userid')
        code = self.request.form.get('code')
        if userid and code:
            exc_key = auth.get_country_exception_cache_key(userid)
            try:
                data = cache.get(exc_key)
                if not strings_differ(data['code'], code):
                    timestamp = data.get('timestamp')
                    if timestamp and (time.time() < (timestamp + (12 * 60 * 60))):
                        user = api.user.get(data['userid'])
                        self.message = 'Successfully issued country login exception for {}({}).'.format(  # noqa
                            user.getProperty('fullname') or user.getUserName(),
                            user.getUserName())
                        self.success = True
                        data['granted'] = True
                        data['timestamp'] = time.time()
                        cache.set(exc_key, data, 12 * 60 * 60)
                        self.send_email(data)
            except Exception:
                pass
        return self.index()

    def send_email(self, user, data):
        email_subject = "Country block exception request granted(Site: %s)" % (
            api.portal.get_registry_record('plone.site_title'))

        email = user.getProperty('email')

        email_data = data.copy()
        email_data.update({
            'name': (user.getProperty('fullname') or user.getUserName())
        })
        email_html = '''
<p>
Hello {name},
</p>
<p>
Your request to login from your current location is granted.
<br />
Please login again with the same browser and location you made the request.
If you still experience any problems, please contact your administrator.

<p>
User and location information:
</p>
<ul>
<li><b>Username</b>: {username}</li>
<li><b>Name</b>: {name}</li>
<li><b>Country</b>: {country}</li>
<li><b>IP</b>: {ip}</li>
</ul>

<p>You have 12 hours to use this granted login exception.</p>
'''.format(**email_data)
        send_email(email, email_subject, html=email_html)



from zope.i18nmessageid import MessageFactory

_ = MessageFactory("collective.pwexpiry")

# Ideally, this should be language dependent, but we'll do US default for now
DATETIME_FORMATSTRING = "%m/%d/%Y %H:%M"



from castle.cms.interfaces import (IAPISettings, IArchivalSettings,
                                   ICastleSettings, IContentSettings,
                                   ISiteConfiguration)
from castle.cms.widgets import FileUploadFieldsFieldWidget, SelectFieldWidget
from plone.app.registry.browser import controlpanel
from plone.formwidget.namedfile.widget import NamedFileFieldWidget
from Products.CMFPlone import PloneMessageFactory as _
from z3c.form import field, group
from z3c.form.interfaces import HIDDEN_MODE


class APIForm(group.GroupForm):
    label = u"APIs"
    fields = field.Fields(IAPISettings)


class ArchivalForm(group.GroupForm):
    label = u"Archival"
    fields = field.Fields(IArchivalSettings)


class ContentForm(group.GroupForm):
    label = u"Content"
    fields = field.Fields(IContentSettings)


class CastleSettingsControlPanelForm(controlpanel.RegistryEditForm):

    id = "CastleSettingsControlPanel"
    label = _(u"CastleCMS Settings")
    description = "Manage CastleCMS-specific settings"
    schema = ICastleSettings
    schema_prefix = "castle"
    fields = field.Fields(ISiteConfiguration)
    groups = (APIForm, ArchivalForm, ContentForm)

    def updateFields(self):
        super(CastleSettingsControlPanelForm, self).updateFields()
        self.groups[0].fields['google_api_service_key_file'].widgetFactory = NamedFileFieldWidget
        self.groups[1].fields['archival_types_to_archive'].widgetFactory = SelectFieldWidget
        self.groups[1].fields['archival_states_to_archive'].widgetFactory = SelectFieldWidget
        self.groups[2].fields['file_upload_fields'].widgetFactory = FileUploadFieldsFieldWidget

    def update(self):
        super(CastleSettingsControlPanelForm, self).update()
        self.groups[1].widgets["archival_replacements"].mode = HIDDEN_MODE


class CastleSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = CastleSettingsControlPanelForm



from AccessControl.SecurityManagement import newSecurityManager
from castle.cms.linkreporter import Reporter
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton


def run_link_report(site):
    reporter = Reporter(site)
    if not reporter.valid:
        return
    try:
        reporter()
    except KeyboardInterrupt:
        reporter.join()


def run(app):
    singleton.SingleInstance('linkreport')

    user = app.acl_users.getUser('admin')
    newSecurityManager(None, user.__of__(app.acl_users))

    for oid in app.objectIds():
        obj = app[oid]
        if IPloneSiteRoot.providedBy(obj):
            run_link_report(obj)


if __name__ == '__main__':
    run(app)  # noqa



import plone.supermodel.exportimport
from plone.supermodel.utils import valueToElement
from zope.dottedname.resolve import resolve
from zope.schema.interfaces import IBaseVocabulary
from zope.schema.interfaces import IContextSourceBinder

from zope.schema.interfaces import ISource

from .field import QueryChoice


class QueryChoiceHandler(plone.supermodel.exportimport.ChoiceHandler):
    """
    Extend/override ChoiceHandler to work for QueryChoice field
    """

    def _constructField(self, attributes):
        if 'vocabulary' in attributes:
            try:
                component = resolve(attributes['vocabulary'])
                if IBaseVocabulary.providedBy(component):
                    attributes['vocabulary'] = component
                elif (ISource.providedBy(component) or
                        IContextSourceBinder.providedBy(component)):
                    attributes['source'] = component
                    del attributes['vocabulary']
            except ImportError:
                # regular named vocabulary, leave as is
                pass
        return super(QueryChoiceHandler, self)._constructField(attributes)

    def write(self, field, name, type, elementName='field'):
        element = super(plone.supermodel.exportimport.ChoiceHandler, self).write(
            field,
            name,
            type,
            elementName
        )
        if field.vocabulary is not None:
            value = "{}.{}".format(
                field.vocabulary.__module__,
                field.vocabulary.__name__)
            child = valueToElement(
                self.fieldAttributes['vocabulary'],
                value,
                name='vocabulary',
                force=True
            )
            element.append(child)
        else:
            value = "{}.{}".format(
                field.source.__module__,
                field.source.__name__)
            child = valueToElement(
                self.fieldAttributes['source'],
                value,
                name='source',
                force=True
            )
            element.append(child)

        return element


QueryChoiceImportExportHandler = QueryChoiceHandler(QueryChoice)



# -*- coding: utf-8 -*-
import json
import time
import unittest

import responses
from AccessControl import AuthEncoding
from castle.cms import cache
from castle.cms import constants
from castle.cms import security
from castle.cms.browser.security.login import SecureLoginView
from castle.cms.interfaces import IAuthenticator
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms.cron._pw_expiry import update_password_expiry
from DateTime import DateTime
from plone import api
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import logout
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryUtility


try:
    import argon2
except ImportError:
    argon2 = None


SHIELD = constants.SHIELD
TEST_USER_NEW_PASSWORD = TEST_USER_PASSWORD + '123'


class TestTwoFactor(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        registry = queryUtility(IRegistry)
        registry['plone.two_factor_enabled'] = True
        registry['castle.plivo_auth_id'] = u'foobar'

        portal_memberdata = self.portal.portal_memberdata
        if not portal_memberdata.hasProperty("reset_password_required"):
            portal_memberdata.manage_addProperty(
                id="reset_password_required", value=False, type="boolean")
            portal_memberdata.manage_addProperty(
                id="reset_password_time", value=time.time(), type="float")
        logout()
        # mock the session id and login session cookie
        self.request.cookies['castle_session_id'] = 'test_session'
        self.request.cookies['__sl__'] = 'test_login'
        self.auth = self.authenticator = getMultiAdapter(
            (self.portal, self.request), IAuthenticator)
        user = api.user.get(username=TEST_USER_NAME)
        user.setMemberProperties(mapping={'email': 'foo@bar.com', })

    def test_authentication_adapter(self):
        getMultiAdapter((self.portal, self.request), IAuthenticator)

    def test_get_options(self):
        view = SecureLoginView(self.portal, self.request)
        opts = json.loads(view.options())
        self.assertTrue(opts['twoFactorEnabled'])
        self.assertEquals(len(opts['supportedAuthSchemes']), 2)

    def test_success_url_is_dashboard(self):
        view = SecureLoginView(self.portal, self.request)
        opts = json.loads(view.options())
        self.assertTrue('@@dashboard' in opts['successUrl'])

    def test_success_url_uses_came_from(self):
        self.request.form.update({
            'came_from': self.portal.absolute_url() + '/foobar'
        })
        view = SecureLoginView(self.portal, self.request)
        opts = json.loads(view.options())
        self.assertTrue('/foobar' in opts['successUrl'])

    @responses.activate
    def test_send_text_message_with_code(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        responses.add(
            responses.POST,
            "https://api.plivo.com/v1/Account/foobar_auth_id/Message/",
            body='{"success": true}',
            content_type="application/json")

        registry = queryUtility(IRegistry)
        registry['castle.plivo_auth_id'] = u'foobar_auth_id'
        registry['castle.plivo_auth_token'] = u'foobar_auth_token'
        registry['castle.plivo_phone_number'] = u'15555555555'

        user = api.user.get(username=TEST_USER_NAME)
        user.setMemberProperties(mapping={'phone_number': '19999999999', })
        self.request.form.update({
            'authType': 'sms',
            'username': TEST_USER_NAME
        })

        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])
        self.assertEquals(len(responses.calls), 1)
        text_body = json.loads(responses.calls[0].request.body)
        self.assertTrue('code:' in text_body['text'])
        self.assertEquals(text_body['dst'], '19999999999')
        self.assertEquals(text_body['src'], '15555555555')

    def test_send_email_with_code(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state

        self.request.form.update({
            'authType': 'email',
            'username': TEST_USER_NAME
        })

        self.request.REQUEST_METHOD = 'POST'
        json.loads(view())
        mailhost = self.portal.MailHost
        self.assertEqual(len(mailhost.messages), 1)

    def test_authorize_code_succeeds(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        self.request.form.update({
            'authType': 'email',
            'username': TEST_USER_NAME
        })
        self.request.REQUEST_METHOD = 'POST'
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        code = cache.get(view.auth.get_2factor_code_key(TEST_USER_NAME))['code']
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD,
            'code': code
        })

        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

    def test_authorize_code_fails(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        self.request.form.update({
            'authType': 'email',
            'username': TEST_USER_NAME
        })
        self.request.REQUEST_METHOD = 'POST'
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD,
            'code': 'foobar'
        })

        result = json.loads(view())
        self.assertFalse(result['success'])

    def test_authorize_code_required_for_login(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        self.request.form.update({
            'authType': 'email',
            'username': TEST_USER_NAME
        })
        self.request.REQUEST_METHOD = 'POST'
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        result = json.loads(view())
        self.assertFalse(result['success'])

    def test_two_factor_login_success(self):
        registry = getUtility(IRegistry)
        registry['plone.two_factor_enabled'] = True
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        self.request.form.update({
            'authType': 'email',
            'username': TEST_USER_NAME
        })
        self.request.REQUEST_METHOD = 'POST'
        view()  # CHECK_CREDENTIALS state
        code = cache.get(view.auth.get_2factor_code_key(TEST_USER_NAME))['code']
        self.auth.set_secure_flow_state(self.auth.CHECK_CREDENTIALS)
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD,
            'code': code
        })

        result = json.loads(view())
        self.assertTrue(result['success'])

    def test_two_factor_login_failure(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        self.request.form.update({
            'username': TEST_USER_NAME
        })
        self.request.REQUEST_METHOD = 'POST'
        view()  # CHECK_CREDENTIALS state
        code = cache.get(view.auth.get_2factor_code_key(TEST_USER_NAME))['code']
        self.auth.set_secure_flow_state(self.auth.CHECK_CREDENTIALS)
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': 'foobar',
            'code': code
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertFalse(result['success'])

    def test_login_success_without_two_factor(self):
        registry = getUtility(IRegistry)
        registry['plone.two_factor_enabled'] = False
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

    def test_authorize_code_does_not_work_out_of_time(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # REQUESTING_AUTH_CODE state
        self.request.form.update({
            'username': TEST_USER_NAME
        })
        self.request.REQUEST_METHOD = 'POST'
        view()  # CHECK_CREDENTIALS state
        code = cache.get(view.auth.get_2factor_code_key(TEST_USER_NAME))['code']
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD,
            'code': code
        })

        # set the code back
        cache_key = view.auth.get_2factor_code_key(TEST_USER_NAME)
        code_data = cache.get(cache_key)
        code_data['timestamp'] -= (5 * 60) + 1
        cache.set(cache_key, code_data)

        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertFalse(result['success'])

    def test_password_expired_does_not_allow_login(self):
        user = api.user.get(username=TEST_USER_NAME)
        user.setMemberProperties(mapping={
            'reset_password_required': True,
            'reset_password_time': time.time() - (48 * 60 * 60)
        })

        registry = getUtility(IRegistry)
        registry['plone.two_factor_enabled'] = False

        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state

        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertFalse(result['success'])

    def test_password_expired(self):
        user = api.user.get(username=TEST_USER_NAME)
        user.setMemberProperties(mapping={
            'reset_password_required': True,
            'reset_password_time': time.time()
        })

        registry = getUtility(IRegistry)
        registry['plone.two_factor_enabled'] = False

        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state

        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD,
            '_authenticator': createToken()
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])
        self.assertTrue(result['changePasswordRequired'])

    # These "reset" tests were actually testing the removed pwexpiry form...
    # def test_password_reset(self):
    #     registry = getUtility(IRegistry)
    #     registry['plone.two_factor_enabled'] = False
    #     self.request.form.update({
    #         'username': TEST_USER_NAME,
    #         'existing_password': TEST_USER_PASSWORD,
    #         'new_password': TEST_USER_NEW_PASSWORD,
    #         '_authenticator': createToken()
    #     })
    #     self.request.method = self.request.REQUEST_METHOD = 'POST'
    #     view = PasswordResetView(self.portal, self.request)
    #     result = json.loads(view())
    #     self.assertTrue(result['success'])
    #
    # def test_password_reset_password_does_not_match(self):
    #     registry = getUtility(IRegistry)
    #     registry['plone.two_factor_enabled'] = False
    #     login(self.portal, TEST_USER_NAME)
    #     self.request.form.update({
    #         'username': TEST_USER_NAME,
    #         'existing_password': 'foobar',
    #         'new_password': 'foobar2',
    #         '_authenticator': createToken()
    #     })
    #     self.request.method = self.request.REQUEST_METHOD = 'POST'
    #     view = PasswordResetView(self.portal, self.request)
    #     result = json.loads(view())
    #     self.assertFalse(result['success'])

    def test_country_code_not_allowed(self):
        registry = getUtility(IRegistry)
        registry['plone.two_factor_enabled'] = False
        registry['plone.restrict_logins_to_countries'] = (u'US',)
        self.request.environ['HTTP_CF_IPCOUNTRY'] = 'AF'

        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state

        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertFalse(result['success'])
        self.assertTrue(result['countryBlocked'])


class TestEnforceBackendEditingUrl(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        logout()
        self.request.cookies['castle_session_id'] = 'test_session'
        self.request.cookies['__sl__'] = 'test_login'
        self.auth = self.authenticator = getMultiAdapter(
            (self.portal, self.request), IAuthenticator)

    def test_setting_disabled(self):
        api.portal.set_registry_record(
            name='plone.only_allow_login_to_backend_urls',
            value=False
        )
        api.portal.set_registry_record(
            name='plone.backend_url',
            value=(unicode(''),)
        )
        view = SecureLoginView(self.portal, self.request)
        view()  # call the view once to set initial state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

    def test_setting_enabled_bad_url(self):
        api.portal.set_registry_record(
            name='plone.only_allow_login_to_backend_urls',
            value=True
        )
        api.portal.set_registry_record(
            name='plone.backend_url',
            value=(unicode(''),)
        )
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertFalse(result['success'])

    def test_setting_enabled_good_url(self):
        api.portal.set_registry_record(
            name='plone.only_allow_login_to_backend_urls',
            value=True
        )
        api.portal.set_registry_record(
            name='plone.backend_url',
            value=(
                unicode('http://dummydomain/castle'),
                unicode('http://nohost/plone'),
                unicode('http://vpn.example.com')
            )
        )
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

    def test_setting_enabled_no_urls(self):
        api.portal.set_registry_record(
            name='plone.only_allow_login_to_backend_urls',
            value=True
        )
        api.portal.set_registry_record(
            name='plone.backend_url',
            value=()
        )
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

# TODO set_password removed from login view..test @@change-password?
# class TestPasswordLength(unittest.TestCase):
#     layer = CASTLE_PLONE_INTEGRATION_TESTING
#
#     def setUp(self):
#         self.portal = self.layer['portal']
#         self.request = self.layer['request']
#         login(self.portal, TEST_USER_NAME)
#
#     def test_short_password(self):
#         shortpass = 'pass'
#         self.request.form.update({
#             'apiMethod': 'set_password',
#             'username': TEST_USER_NAME,
#             'existing_password': TEST_USER_PASSWORD,
#             'new_password': shortpass
#         })
#         view = SecureLoginView(self.portal, self.request)
#         result = json.loads(view())
#         self.assertFalse(result['success'])
#
#     def test_long_password(self):
#         longpass = 'N1C3P@$$w0rd'
#         self.request.form.update({
#             'apiMethod': 'set_password',
#             'username': TEST_USER_NAME,
#             'existing_password': TEST_USER_PASSWORD,
#             'new_password': longpass
#         })
#         view = SecureLoginView(self.portal, self.request)
#         result = json.loads(view())
#         self.assertTrue(result['success'])


class TestPwexpiry(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        api.portal.set_registry_record(
            name='plone.pwexpiry_enabled',
            value=True
        )
        api.portal.set_registry_record(
            name='plone.pwexpiry_whitelisted_users',
            value=[]
        )
        logout()
        self.request.cookies['castle_session_id'] = 'test_session'
        self.request.cookies['__sl__'] = 'test_login'
        self.auth = self.authenticator = getMultiAdapter(
            (self.portal, self.request), IAuthenticator)

    def test_initial_login(self):
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

    def test_expired_login(self):
        editableUser = api.user.get(username=TEST_USER_NAME)
        editableUser.setMemberProperties({
            'password_date': DateTime('01/10/2011')
        })
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        update_password_expiry(self.portal)
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['changePasswordRequired'])

    def test_whitelist(self):
        editableUser = api.user.get(username=TEST_USER_NAME)
        editableUser.setMemberProperties({
            'password_date': DateTime('01/10/2011')
        })
        api.portal.set_registry_record(
            name='plone.pwexpiry_whitelisted_users',
            value=[editableUser.getId().decode('utf-8')]
        )
        view = SecureLoginView(self.portal, self.request)
        view()  # CHECK_CREDENTIALS state
        self.request.form.update({
            'username': TEST_USER_NAME,
            'password': TEST_USER_PASSWORD
        })
        self.request.REQUEST_METHOD = 'POST'
        result = json.loads(view())
        self.assertTrue(result['success'])

    # Again for now there is no longer a change-password without being authenticatedself.
    # To test history would have to add tests for the @@change-password view
    # def test_password_history(self):
    #     pass1 = 'N1C3P@$$w0rd'
    #     pass2 = 'P@$$w0rd2018'
    #     editableUser = api.user.get(username=TEST_USER_NAME)
    #     editableUser.setMemberProperties({
    #         'password_date': DateTime('01/10/2011')
    #     })
    #     login(self.portal, TEST_USER_NAME)
    #
    #     # -----Change password-----
    #     self.request.form.update({
    #         'apiMethod': 'set_password',
    #         'username': TEST_USER_NAME,
    #         'existing_password': TEST_USER_PASSWORD,
    #         'new_password': pass1
    #     })
    #     view = SecureLoginView(self.portal, self.request)
    #     result = json.loads(view())
    #     self.assertTrue(result['success'])
    #     logout()
    #
    #     # -----Try logging in with new password-----
    #     self.request.form.update({
    #         'apiMethod': 'login',
    #         'username': TEST_USER_NAME,
    #         'password': pass1
    #     })
    #     view = SecureLoginView(self.portal, self.request)
    #     result = json.loads(view())
    #     self.assertFalse(result['resetpassword'])
    #
    #     # -----Change password again-----
    #     login(self.portal, TEST_USER_NAME)
    #     self.request.form.update({
    #         'apiMethod': 'set_password',
    #         'username': TEST_USER_NAME,
    #         'existing_password': pass1,
    #         'new_password': pass2
    #     })
    #     view = SecureLoginView(self.portal, self.request)
    #     result = json.loads(view())
    #     self.assertTrue(result['success'])
    #
    #     # -----Try to change password back-----
    #     self.request.form.update({
    #         'apiMethod': 'set_password',
    #         'username': TEST_USER_NAME,
    #         'existing_password': pass2,
    #         'new_password': pass1
    #     })
    #     view = SecureLoginView(self.portal, self.request)
    #     result = json.loads(view())
    #     self.assertFalse(result['success'])


if argon2 is not None:
    class TestArgon2(unittest.TestCase):
        def test_registered(self):
            self.assertTrue('argon2' in [s[0] for s in AuthEncoding._schemes])

        def test_encrypt(self):
            scheme = security.Argon2Scheme()
            encrypted = scheme.encrypt('foobar')
            self.assertTrue(scheme.validate(encrypted, 'foobar'))

        def test_argon_is_used_by_default(self):
            encrypted = AuthEncoding.pw_encrypt('foobar')
            self.assertTrue('{argon2}' in encrypted)
            self.assertTrue(AuthEncoding.pw_validate(encrypted, 'foobar'))



import json
from Products.Five.browser import BrowserView
from castle.cms import utils


class ImageInfoView(BrowserView):

    def __call__(self):
        response = self.request.response
        response.setHeader("Content-type", "application/json")
        return json.dumps(utils.get_image_info(self.context))



# -*- coding: utf-8 -*-

from castle.cms.interfaces.passwordvalidation import (IInvalidPasswordEntered,
                                                      IUserUnlocked,
                                                      IValidPasswordEntered)
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class ValidPasswordEntered(ObjectEvent):
    """A user entered a valid password
    """
    implements(IValidPasswordEntered)


class InvalidPasswordEntered(ObjectEvent):
    """A user entered an invalid password
    """
    implements(IInvalidPasswordEntered)


class UserUnlocked(object):
    """A user has been unlocked from the control panel tool
    """
    implements(IUserUnlocked)

    def __init__(self, user):
        self.user = user



from castle.cms.tiles.base import BaseTile
from castle.cms.widgets import ImageRelatedItemFieldWidget
from castle.cms.widgets import VideoRelatedItemsFieldWidget
from plone.supermodel import model
from plone.autoform import directives as form
from zope import schema
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class SlideTile(BaseTile):

    @property
    def slide_type(self):
        return self.data.get('display_type', 'background-image')


class ISlideTileSchema(model.Schema):
    display_type = schema.Choice(
        title=u"Display Type",
        vocabulary=SimpleVocabulary([
            SimpleTerm('background-image', 'background-image', u'Background Image'),  # noqa
            SimpleTerm('left-image-right-text','left-image-right-text', u'Left Image Right Text'),  # noqa
            SimpleTerm('background-video', 'background-video', u'Background Video'),  # noqa
            SimpleTerm('left-video-right-text','left-video-right-text', u'Left Video Right Text')  # noqa
        ]),
        default='background-image'
    )

    title = schema.TextLine(
        title=u'Slide Title',
        description=u'Will be omitted if blank',
        required=False,
        default=u'')

    text = schema.Text(
        title=u'Slide Text',
        description=u'Will be omitted if blank',
        required=False,
        default=u'')

    vert_text_position = schema.Choice(
        title=u"Slide Text Position (Vertical)",
        vocabulary=SimpleVocabulary([
            SimpleTerm('top', 'top', u'Top'),
            SimpleTerm('middle', 'middle', u'Middle'),
            SimpleTerm('bottom', 'bottom', u'Bottom'),
        ]),
        required=True,
        default=u'middle'
    )

    hor_text_position = schema.Choice(
        title=u"Slide Text Position (Horizontal)",
        vocabulary=SimpleVocabulary([
            SimpleTerm('start', 'start', u'Left'),
            SimpleTerm('center', 'center', u'Center'),
            SimpleTerm('end', 'end', u'Right'),
        ]),
        required=True,
        default=u'center'
    )

    form.widget(image=ImageRelatedItemFieldWidget)
    image = schema.List(
        title=u"Image",
        description=u"Reference image on the site.",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    form.widget(video=VideoRelatedItemsFieldWidget)
    video = schema.List(
        title=u"Video",
        description=u"Reference video on the site.",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )
